@@ -1,18 +1,22 @@
 package Zonemaster::Engine::Test::Consistency;
 
-use version; our $VERSION = version->declare("v1.1.8");
+use 5.014002;
 
 use strict;
 use warnings;
 
-use 5.014002;
+use version; our $VERSION = version->declare("v1.1.15");
 
 use Zonemaster::Engine;
-use Zonemaster::Engine::Util;
-use Zonemaster::Engine::Test::Address;
-use Zonemaster::Engine::Constants qw[:ip :soa];
 
 use List::MoreUtils qw[uniq];
+use Locale::TextDomain qw[Zonemaster-Engine];
+use Readonly;
+use Zonemaster::Engine::Profile;
+use Zonemaster::Engine::Constants qw[:ip :soa];
+use Zonemaster::Engine::Test::Address;
+use Zonemaster::Engine::Util;
+use Zonemaster::Engine::TestMethods;
 
 ###
 ### Entry points
@@ -62,6 +66,8 @@ sub metadata {
               SOA_SERIAL_VARIATION
               IPV4_DISABLED
               IPV6_DISABLED
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         consistency02 => [
@@ -73,6 +79,8 @@ sub metadata {
               SOA_RNAME
               IPV4_DISABLED
               IPV6_DISABLED
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         consistency03 => [
@@ -84,6 +92,8 @@ sub metadata {
               SOA_TIME_PARAMETER_SET
               IPV4_DISABLED
               IPV6_DISABLED
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         consistency04 => [
@@ -95,6 +105,8 @@ sub metadata {
               NS_SET
               IPV4_DISABLED
               IPV6_DISABLED
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         consistency05 => [
@@ -106,6 +118,8 @@ sub metadata {
               IN_BAILIWICK_ADDR_MISMATCH
               NO_RESPONSE
               OUT_OF_BAILIWICK_ADDR_MISMATCH
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         consistency06 => [
@@ -114,42 +128,143 @@ sub metadata {
               NO_RESPONSE_SOA_QUERY
               ONE_SOA_MNAME
               MULTIPLE_SOA_MNAMES
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
     };
 } ## end sub metadata
 
-sub translation {
-    return {
-        ADDRESSES_MATCH      => 'Glue records are consistent between glue and authoritative data.',
-        EXTRA_ADDRESS_CHILD  => 'Child has extra nameserver IP address(es) not listed at parent ({addresses}).',
-        EXTRA_ADDRESS_PARENT => 'Parent has extra nameserver IP address(es) not listed at child ({addresses}).',
-        IPV4_DISABLED        => 'IPv4 is disabled, not sending "{rrtype}" query to {ns}/{address}.',
-        IPV6_DISABLED        => 'IPv6 is disabled, not sending "{rrtype}" query to {ns}/{address}.',
-        MULTIPLE_NS_SET      => 'Saw {count} NS set.',
-        MULTIPLE_SOA_MNAMES  => 'Saw {count} SOA mname.',
-        MULTIPLE_SOA_RNAMES  => 'Saw {count} SOA rname.',
-        MULTIPLE_SOA_SERIALS => 'Saw {count} SOA serial numbers.',
-        MULTIPLE_SOA_TIME_PARAMETER_SET => 'Saw {count} SOA time parameter set.',
-        NO_RESPONSE                     => 'Nameserver {ns}/{address} did not respond.',
-        NO_RESPONSE_NS_QUERY            => 'No response from nameserver {ns}/{address} on NS queries.',
-        NO_RESPONSE_SOA_QUERY           => 'No response from nameserver {ns}/{address} on SOA queries.',
-        NS_SET                          => 'Saw NS set ({nsset}) on following nameserver set : {servers}.',
-        ONE_NS_SET                      => 'A unique NS set was seen ({nsset}).',
-        ONE_SOA_MNAME                   => 'A single SOA mname value was seen ({mname})',
-        ONE_SOA_RNAME                   => 'A single SOA rname value was seen ({rname})',
-        ONE_SOA_SERIAL                  => 'A single SOA serial number was seen ({serial}).',
-        ONE_SOA_TIME_PARAMETER_SET      => 'A single SOA time parameter set was seen '
+Readonly my %TAG_DESCRIPTIONS => (
+    ADDRESSES_MATCH => sub {
+        __x    # CONSISTENCY:ADDRESSES_MATCH
+          'Glue records are consistent between glue and authoritative data.', @_;
+    },
+    CHILD_NS_FAILED => sub {
+        __x    # CONSISTENCY:CHILD_NS_FAILED
+          'Unexpected or erroneous reply from {ns}.', @_;
+    },
+    CHILD_ZONE_LAME => sub {
+        __x    # CONSISTENCY:CHILD_ZONE_LAME
+          'Lame delegation.', @_;
+    },
+    EXTRA_ADDRESS_CHILD => sub {
+        __x    # CONSISTENCY:EXTRA_ADDRESS_CHILD
+          'Child has extra nameserver IP address(es) not listed at parent ({ns_ip_list}).', @_;
+    },
+    IN_BAILIWICK_ADDR_MISMATCH => sub {
+        __x    # CONSISTENCY:IN_BAILIWICK_ADDR_MISMATCH
+          'In-bailiwick name server listed at parent has a mismatch between glue data at parent '
+          . '({parent_addresses}) and any equivalent address record in child zone ({zone_addresses}).',
+          @_;
+    },
+    IPV4_DISABLED => sub {
+        __x    # CONSISTENCY:IPV4_DISABLED
+          'IPv4 is disabled, not sending "{rrtype}" query to {ns}.', @_;
+    },
+    IPV6_DISABLED => sub {
+        __x    # CONSISTENCY:IPV6_DISABLED
+          'IPv6 is disabled, not sending "{rrtype}" query to {ns}.', @_;
+    },
+    MULTIPLE_NS_SET => sub {
+        __x    # CONSISTENCY:MULTIPLE_NS_SET
+          'Found {count} NS set(s).', @_;
+    },
+    MULTIPLE_SOA_MNAMES => sub {
+        __x    # CONSISTENCY:MULTIPLE_SOA_MNAMES
+          'Saw {count} SOA mname.', @_;
+    },
+    MULTIPLE_SOA_RNAMES => sub {
+        __x    # CONSISTENCY:MULTIPLE_SOA_RNAMES
+          'Found {count} SOA rname(s).', @_;
+    },
+    MULTIPLE_SOA_SERIALS => sub {
+        __x    # CONSISTENCY:MULTIPLE_SOA_SERIALS
+          'Found {count} SOA serial number(s).', @_;
+    },
+    MULTIPLE_SOA_TIME_PARAMETER_SET => sub {
+        __x    # CONSISTENCY:MULTIPLE_SOA_TIME_PARAMETER_SET
+          "Found {count} SOA time parameter set(s).", @_;
+    },
+    NO_RESPONSE => sub {
+        __x    # CONSISTENCY:NO_RESPONSE
+          'Nameserver {ns} did not respond.', @_;
+    },
+    NO_RESPONSE_NS_QUERY => sub {
+        __x    # CONSISTENCY:NO_RESPONSE_NS_QUERY
+          'No response from nameserver {ns} on NS queries.', @_;
+    },
+    NO_RESPONSE_SOA_QUERY => sub {
+        __x    # CONSISTENCY:NO_RESPONSE_SOA_QUERY
+          'No response from nameserver {ns} on SOA queries.', @_;
+    },
+    NS_SET => sub {
+        __x    # CONSISTENCY:NS_SET
+          'Saw NS set ({nsname_list}) on following nameserver set : {servers}.', @_;
+    },
+    ONE_NS_SET => sub {
+        __x    # CONSISTENCY:ONE_NS_SET
+          "A single NS set was found ({nsname_list}).", @_;
+    },
+    ONE_SOA_MNAME => sub {
+        __x    # CONSISTENCY:ONE_SOA_MNAME
+          "A single SOA mname value was seen ({mname}).", @_;
+    },
+    ONE_SOA_RNAME => sub {
+        __x    # CONSISTENCY:ONE_SOA_RNAME
+          "A single SOA rname value was found ({rname}).", @_;
+    },
+    ONE_SOA_SERIAL => sub {
+        __x    # CONSISTENCY:ONE_SOA_SERIAL
+          "A single SOA serial number was found ({serial}).", @_;
+    },
+    ONE_SOA_TIME_PARAMETER_SET => sub {
+        __x    # CONSISTENCY:ONE_SOA_TIME_PARAMETER_SET
+          'A single SOA time parameter set was seen '
           . '(REFRESH={refresh},RETRY={retry},EXPIRE={expire},MINIMUM={minimum}).',
-        SOA_RNAME            => 'Saw SOA rname {rname} on following nameserver set : {servers}.',
-        SOA_SERIAL           => 'Saw SOA serial number {serial} on following nameserver set : {servers}.',
-        SOA_SERIAL_VARIATION => 'Difference between the smaller serial '
-          . '({serial_min}) and the bigger one ({serial_max}) is greater than the maximum allowed ({max_variation}).',
-        SOA_TIME_PARAMETER_SET => 'Saw SOA time parameter set '
-          . '(REFRESH={refresh},RETRY={retry},EXPIRE={expire},MINIMUM={minimum}) on following nameserver set : {servers}.',
-        TOTAL_ADDRESS_MISMATCH => 'No common nameserver IP addresses between child ({child}) and parent ({glue}).',
-    };
-} ## end sub translation
+          @_;
+    },
+    OUT_OF_BAILIWICK_ADDR_MISMATCH => sub {
+        __x    # CONSISTENCY:OUT_OF_BAILIWICK_ADDR_MISMATCH
+          'Out-of-bailiwick name server listed at parent with glue record has a mismatch between '
+          . 'the glue at the parent ({parent_addresses}) and any equivalent address record found '
+          . 'in authoritative zone ({zone_addresses}).', @_;
+    },
+    SOA_RNAME => sub {
+        __x    # CONSISTENCY:SOA_RNAME
+          "Found SOA rname {rname} on following nameserver set : {ns_list}.", @_;
+    },
+    SOA_SERIAL => sub {
+        __x    # CONSISTENCY:SOA_SERIAL
+          'Saw SOA serial number {serial} on following nameserver set : {ns_list}.', @_;
+    },
+    SOA_SERIAL_VARIATION => sub {
+        __x    # CONSISTENCY:SOA_SERIAL_VARIATION
+          'Difference between the smaller serial ({serial_min}) and the bigger one ({serial_max}) '
+          . 'is greater than the maximum allowed ({max_variation}).', @_;
+    },
+    SOA_TIME_PARAMETER_SET => sub {
+        __x    # CONSISTENCY:SOA_TIME_PARAMETER_SET
+          'Saw SOA time parameter set (REFRESH={refresh},RETRY={retry},EXPIRE={expire},'
+          . 'MINIMUM={minimum}) on following nameserver set : {ns_list}.', @_;
+    },
+    TEST_CASE_END => sub {
+        __x    # CONSISTENCY:TEST_CASE_END
+          'TEST_CASE_END {testcase}.', @_;
+    },
+    TEST_CASE_START => sub {
+        __x    # CONSISTENCY:TEST_CASE_START
+          'TEST_CASE_START {testcase}.', @_;
+    },
+    TOTAL_ADDRESS_MISMATCH => sub {
+        __x    # CONSISTENCY:TOTAL_ADDRESS_MISMATCH
+          'No common nameserver IP addresses between child ({child}) and parent ({glue}).', @_;
+    },
+);
+
+sub tag_descriptions {
+    return \%TAG_DESCRIPTIONS;
+}
 
 sub version {
     return "$Zonemaster::Engine::Test::Consistency::VERSION";
@@ -161,7 +276,7 @@ sub version {
 
 sub consistency01 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my %nsnames_and_ip;
     my %serials;
     my $query_type = q{SOA};
@@ -176,9 +291,8 @@ sub consistency01 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -188,9 +302,8 @@ sub consistency01 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -199,26 +312,14 @@ sub consistency01 {
         my $p = $local_ns->query( $zone->name, $query_type );
 
         if ( not $p ) {
-            push @results,
-              info(
-                NO_RESPONSE => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                }
-              );
+            push @results, info( NO_RESPONSE => { ns => $local_ns->string } );
             next;
         }
 
         my ( $soa ) = $p->get_records_for_name( $query_type, $zone->name );
 
         if ( not $soa ) {
-            push @results,
-              info(
-                NO_RESPONSE_SOA_QUERY => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                }
-              );
+            push @results, info( NO_RESPONSE_SOA_QUERY => { ns => $local_ns->string } );
             next;
         }
         else {
@@ -234,7 +335,7 @@ sub consistency01 {
           info(
             SOA_SERIAL => {
                 serial  => $serial,
-                servers => join( q{;}, sort @{ $serials{$serial} } ),
+                ns_list => join( q{;}, sort @{ $serials{$serial} } ),
             }
           );
     }
@@ -266,12 +367,12 @@ sub consistency01 {
         }
     } ## end elsif ( scalar @serial_numbers)
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub consistency01
 
 sub consistency02 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my %nsnames_and_ip;
     my %rnames;
     my $query_type = q{SOA};
@@ -286,9 +387,8 @@ sub consistency02 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -298,9 +398,8 @@ sub consistency02 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -309,26 +408,14 @@ sub consistency02 {
         my $p = $local_ns->query( $zone->name, $query_type );
 
         if ( not $p ) {
-            push @results,
-              info(
-                NO_RESPONSE => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                }
-              );
+            push @results, info( NO_RESPONSE => { ns => $local_ns->string } );
             next;
         }
 
         my ( $soa ) = $p->get_records_for_name( $query_type, $zone->name );
 
         if ( not $soa ) {
-            push @results,
-              info(
-                NO_RESPONSE_SOA_QUERY => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                }
-              );
+            push @results, info( NO_RESPONSE_SOA_QUERY => { ns => $local_ns->string } );
             next;
         }
         else {
@@ -357,18 +444,18 @@ sub consistency02 {
               info(
                 SOA_RNAME => {
                     rname   => $rname,
-                    servers => join( q{;}, @{ $rnames{$rname} } ),
+                    ns_list => join( q{;}, @{ $rnames{$rname} } ),
                 }
               );
         }
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub consistency02
 
 sub consistency03 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my %nsnames_and_ip;
     my %time_parameter_sets;
     my $query_type = q{SOA};
@@ -383,9 +470,8 @@ sub consistency03 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -395,9 +481,8 @@ sub consistency03 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -406,26 +491,14 @@ sub consistency03 {
         my $p = $local_ns->query( $zone->name, $query_type );
 
         if ( not $p ) {
-            push @results,
-              info(
-                NO_RESPONSE => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                }
-              );
+            push @results, info( NO_RESPONSE => { ns => $local_ns->string } );
             next;
         }
 
         my ( $soa ) = $p->get_records_for_name( $query_type, $zone->name );
 
         if ( not $soa ) {
-            push @results,
-              info(
-                NO_RESPONSE_SOA_QUERY => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                }
-              );
+            push @results, info( NO_RESPONSE_SOA_QUERY => { ns => $local_ns->string } );
             next;
         }
         else {
@@ -465,18 +538,18 @@ sub consistency03 {
                     retry   => $retry,
                     expire  => $expire,
                     minimum => $minimum,
-                    servers => join( q{;}, sort @{ $time_parameter_sets{$time_parameter_set} } ),
+                    ns_list => join( q{;}, sort @{ $time_parameter_sets{$time_parameter_set} } ),
                 }
               );
         }
     } ## end elsif ( scalar( keys %time_parameter_sets...))
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub consistency03
 
 sub consistency04 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my %nsnames_and_ip;
     my %ns_sets;
     my $query_type = q{NS};
@@ -491,9 +564,8 @@ sub consistency04 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -503,9 +575,8 @@ sub consistency04 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -514,41 +585,24 @@ sub consistency04 {
         my $p = $local_ns->query( $zone->name, $query_type );
 
         if ( not $p ) {
-            push @results,
-              info(
-                NO_RESPONSE => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                }
-              );
+            push @results, info( NO_RESPONSE => { ns => $local_ns->string } );
             next;
         }
 
         my ( @ns ) = sort map { lc( $_->nsdname ) } $p->get_records_for_name( $query_type, $zone->name );
 
         if ( not scalar( @ns ) ) {
-            push @results,
-              info(
-                NO_RESPONSE_NS_QUERY => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                }
-              );
+            push @results, info( NO_RESPONSE_NS_QUERY => { ns => $local_ns->string } );
             next;
         }
         else {
-            push @{ $ns_sets{ join( q{,}, @ns ) } }, $local_ns->name->string . q{/} . $local_ns->address->short;
-            $nsnames_and_ip{ $local_ns->name->string . q{/} . $local_ns->address->short }++;
+            push @{ $ns_sets{ join( q{;}, @ns ) } }, $local_ns->string;
+            $nsnames_and_ip{ $local_ns->string }++;
         }
     } ## end foreach my $local_ns ( @{ Zonemaster::Engine::TestMethods...})
 
     if ( scalar( keys %ns_sets ) == 1 ) {
-        push @results,
-          info(
-            ONE_NS_SET => {
-                nsset => ( keys %ns_sets )[0],
-            }
-          );
+        push @results, info( ONE_NS_SET => { nsname_list => ( keys %ns_sets )[0] });
     }
     elsif ( scalar( keys %ns_sets ) ) {
         push @results,
@@ -561,31 +615,26 @@ sub consistency04 {
             push @results,
               info(
                 NS_SET => {
-                    nsset   => $ns_set,
-                    servers => join( q{;}, @{ $ns_sets{$ns_set} } ),
+                    nsname_list => $ns_set,
+                    servers     => join( q{;}, @{ $ns_sets{$ns_set} } ),
                 }
               );
         }
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub consistency04
 
 sub _get_addr_rrs {
     my ( $class, $ns, $name, $qtype ) = @_;
-    my $p = $ns->query( $name, $qtype );
+    my $p = $ns->query( $name, $qtype, { recurse => 0 } );
     if ( !$p ) {
-        return info(
-            NO_RESPONSE => {
-                ns      => $ns->name->string,
-                address => $ns->address->short,
-            }
-        );
+        return info( NO_RESPONSE => { ns => $ns->string } );
     }
     elsif ($p->is_redirect) {
-        my $p_pub = Zonemaster::Engine->recurse( $name, $qtype, 'IN' );
-        if ( $p_pub ) {
-            return ( undef, $p_pub->get_records_for_name( $qtype, $name, 'answer' ) );
+        my $p = Zonemaster::Engine->recurse( $name, $qtype, q{IN} );
+        if ( $p ) {
+            return ( undef, $p->get_records_for_name( $qtype, $name, 'answer' ) );
         } else {
             return ( undef );
         }
@@ -594,43 +643,64 @@ sub _get_addr_rrs {
         return ( undef, $p->get_records_for_name( $qtype, $name, 'answer' ) );
     }
     elsif (not ($p->aa and $p->rcode eq 'NXDOMAIN')) {
-        return info(
-            CHILD_NS_FAILED => {
-                ns      => $ns->name->string,
-                address => $ns->address->short,
-            }
-        );
+        return info( CHILD_NS_FAILED => { ns => $ns->string } );
     }
+    return ( undef );
 }
 
 sub consistency05 {
     my ( $class, $zone ) = @_;
-    my @results;
-
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my %strict_glue;
     my %extended_glue;
-    for my $ns ( @{ Zonemaster::Engine::TestMethods->method4( $zone ) } ) {
-        my $ns_string = $ns->name->fqdn . "/" . $ns->address->short;
-        if ( $zone->name->is_in_bailiwick( $ns->name ) ) {
+
+    # We need to work on Methods...
+    # This part of code is supposed to replace method4 call
+    my @child_nsnames;
+    my @nsnames;
+    my $ns_aref = $zone->parent->query_all( $zone->name, q{NS} );
+    my %parent_glues;
+    foreach my $p ( @{$ns_aref} ) {
+        next if not $p;
+        push @nsnames, $p->get_records_for_name( q{NS}, $zone->name );
+    }
+    @child_nsnames = uniq map { name( lc( $_->nsdname ) ) } @nsnames;
+    foreach my $nsname ( @child_nsnames ) {
+        my $a_aref = $zone->parent->query_all( $nsname, q{A} );
+        my $aaaa_aref = $zone->parent->query_all( $nsname, q{AAAA} );
+        foreach my $p ( @{$a_aref} ) {
+            next if not $p;
+            foreach my $rr ( $p->get_records_for_name( q{A}, $nsname ) ) {
+                $parent_glues{ lc( $rr->owner ) . q{/} . $rr->address } = $nsname;
+            }
+        }
+        foreach my $p ( @{$aaaa_aref} ) {
+            next if not $p;
+            foreach my $rr ( $p->get_records_for_name( q{AAAA}, $nsname ) ) {
+                $parent_glues{ lc( $rr->owner ) . q{/} . $rr->address } = $nsname;
+            }
+        }
+    }
+
+    for my $ns_string ( keys %parent_glues ) {
+        if ( $zone->name->is_in_bailiwick( $parent_glues{$ns_string} ) ) {
             $strict_glue{ $ns_string } = 1;
         }
         else {
-            push @{ $extended_glue{ $ns->name->string } }, $ns_string;
+            push @{ $extended_glue{ $parent_glues{$ns_string} } }, $ns_string;
         }
     }
 
     my @ib_nsnames =
       grep { $zone->name->is_in_bailiwick( $_ ) } @{ Zonemaster::Engine::TestMethods->method2and3( $zone ) };
-
     my @ib_nss = grep { Zonemaster::Engine::Util::ipversion_ok( $_->address->version ) }
       @{ Zonemaster::Engine::TestMethods->method4and5( $zone ) };
-
     my %child_ib_strings;
     for my $ib_nsname ( @ib_nsnames ) {
         my $is_lame = 1;
         for my $ns ( @ib_nss ) {
-            my ( $msg_a,    @rrs_a )    = $class->_get_addr_rrs( $ns, $ib_nsname, 'A' );
-            my ( $msg_aaaa, @rrs_aaaa ) = $class->_get_addr_rrs( $ns, $ib_nsname, 'AAAA' );
+            my ( $msg_a,    @rrs_a )    = $class->_get_addr_rrs( $ns, $ib_nsname, q{A} );
+            my ( $msg_aaaa, @rrs_aaaa ) = $class->_get_addr_rrs( $ns, $ib_nsname, q{AAAA} );
 
             if ( defined $msg_a ) {
                 push @results, $msg_a;
@@ -643,13 +713,13 @@ sub consistency05 {
             }
 
             for my $rr ( @rrs_a, @rrs_aaaa ) {
-                $child_ib_strings{ lc( $rr->name ) . "/" . $rr->address } = 1;
+                $child_ib_strings{ lc( $rr->name ) . q{/} . $rr->address } = 1;
             }
         }
 
         if ( $is_lame ) {
             push @results, info( CHILD_ZONE_LAME => {} );
-            return @results;
+            return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
         }
     } ## end for my $ib_nsname ( @ib_nsnames)
 
@@ -657,7 +727,7 @@ sub consistency05 {
     my @ib_mismatch    = grep { !exists $child_ib_strings{$_} } keys %strict_glue;
     my @ib_extra_child = grep { !exists $strict_glue{$_} } keys %child_ib_strings;
 
-    if ( @ib_mismatch ) {
+    if ( scalar @ib_mismatch ) {
         push @results,
           info(
             IN_BAILIWICK_ADDR_MISMATCH => {
@@ -666,11 +736,11 @@ sub consistency05 {
             }
           );
     }
-    if ( @ib_extra_child ) {
+    if ( scalar @ib_extra_child ) {
         push @results,
           info(
             EXTRA_ADDRESS_CHILD => {
-                addresses => join( q{;}, sort @ib_extra_child ),
+                ns_ip_list => join( q{;}, sort @ib_extra_child ),
             }
           );
     }
@@ -682,17 +752,17 @@ sub consistency05 {
 
         my %child_oob_strings;
 
-        my $p_a = Zonemaster::Engine->recurse( $glue_name, 'A', 'IN' );
+        my $p_a = Zonemaster::Engine->recurse( $glue_name, q{A}, q{IN} );
         if ( $p_a ) {
-            for my $rr ( $p_a->get_records_for_name( 'A', $glue_name, 'answer' ) ) {
-                $child_oob_strings{ $rr->owner . "/" . $rr->address } = 1;
+            for my $rr ( $p_a->get_records_for_name( q{A}, $glue_name, q{answer} ) ) {
+                $child_oob_strings{ lc( $rr->owner ) . q{/} . $rr->address } = 1;
             }
         }
 
-        my $p_aaaa = Zonemaster::Engine->recurse( $glue_name, 'AAAA', 'IN' );
+        my $p_aaaa = Zonemaster::Engine->recurse( $glue_name, q{AAAA}, q{IN} );
         if ( $p_aaaa ) {
-            for my $rr ( $p_aaaa->get_records_for_name( 'AAAA', $glue_name, 'answer' ) ) {
-                $child_oob_strings{ $rr->owner . "/" . $rr->address } = 1;
+            for my $rr ( $p_aaaa->get_records_for_name( q{AAAA}, $glue_name, q{answer} ) ) {
+                $child_oob_strings{ lc( $rr->owner ) . q{/} . $rr->address } = 1;
             }
         }
 
@@ -710,20 +780,15 @@ sub consistency05 {
     } ## end for my $glue_name ( keys...)
 
     if ( !@ib_extra_child && !@ib_mismatch && !@oob_mismatch ) {
-        push @results,
-          info(
-            ADDRESSES_MATCH => {
-                addresses => join( q{;}, sort @ib_match, @oob_match ),
-            }
-          );
+        push @results, info( ADDRESSES_MATCH => {} );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub consistency05
 
 sub consistency06 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my %nsnames_and_ip;
     my %mnames;
     my $query_type = q{SOA};
@@ -738,9 +803,8 @@ sub consistency06 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -750,9 +814,8 @@ sub consistency06 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -761,26 +824,14 @@ sub consistency06 {
         my $p = $local_ns->query( $zone->name, $query_type );
 
         if ( not $p ) {
-            push @results,
-              info(
-                NO_RESPONSE => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                }
-              );
+            push @results, info( NO_RESPONSE => { ns => $local_ns->string } );
             next;
         }
 
         my ( $soa ) = $p->get_records_for_name( $query_type, $zone->name );
 
         if ( not $soa ) {
-            push @results,
-              info(
-                NO_RESPONSE_SOA_QUERY => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                }
-              );
+            push @results, info( NO_RESPONSE_SOA_QUERY => { ns => $local_ns->string } );
             next;
         }
         else {
@@ -809,13 +860,13 @@ sub consistency06 {
               info(
                 SOA_MNAME => {
                     mname   => $mname,
-                    servers => join( q{;}, @{ $mnames{$mname} } ),
+                    ns_list => join( q{;}, @{ $mnames{$mname} } ),
                 }
               );
         }
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub consistency06
 
 1;
@@ -841,9 +892,9 @@ Runs the default set of tests and returns a list of log entries made by the test
 Returns a reference to a hash, the keys of which are the names of all test methods in the module, and the corresponding values are references to
 lists with all the tags that the method can use in log entries.
 
-=item translation()
+=item tag_descriptions()
 
-Returns a refernce to a hash with translation data. Used by the builtin translation system.
+Returns a refernce to a hash with translation functions. Used by the builtin translation system.
 
 =item version()
 
