@@ -1,21 +1,25 @@
 package Zonemaster::Engine::Test::Delegation;
 
-use version; our $VERSION = version->declare("v1.0.12");
+use 5.014002;
 
 use strict;
 use warnings;
 
-use 5.014002;
+use version; our $VERSION = version->declare("v1.0.19");
 
 use Zonemaster::Engine;
-use Zonemaster::Engine::Util;
+
+use List::MoreUtils qw[uniq];
+use Locale::TextDomain qw[Zonemaster-Engine];
+use Readonly;
+use Zonemaster::Engine::Profile;
+use Zonemaster::Engine::Recursor;
+use Zonemaster::Engine::Constants ':all';
+use Zonemaster::Engine::Net::IP;
 use Zonemaster::Engine::Test::Address;
 use Zonemaster::Engine::Test::Syntax;
 use Zonemaster::Engine::TestMethods;
-use Zonemaster::Engine::Constants ':all';
-use Zonemaster::Engine::Net::IP;
-
-use List::MoreUtils qw[uniq];
+use Zonemaster::Engine::Util;
 use Zonemaster::LDNS::Packet;
 use Zonemaster::LDNS::RR;
 
@@ -53,11 +57,19 @@ sub metadata {
               NOT_ENOUGH_NS_DEL
               NOT_ENOUGH_NS_CHILD
               ENOUGH_IPV4_NS_CHILD
+              ENOUGH_IPV4_NS_DEL
               ENOUGH_IPV6_NS_CHILD
+              ENOUGH_IPV6_NS_DEL
               NOT_ENOUGH_IPV4_NS_CHILD
+              NOT_ENOUGH_IPV4_NS_DEL
               NOT_ENOUGH_IPV6_NS_CHILD
+              NOT_ENOUGH_IPV6_NS_DEL
               NO_IPV4_NS_CHILD
+              NO_IPV4_NS_DEL
               NO_IPV6_NS_CHILD
+              NO_IPV6_NS_DEL
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         delegation02 => [
@@ -68,12 +80,16 @@ sub metadata {
               DEL_NS_SAME_IP
               SAME_IP_ADDRESS
               DISTINCT_IP_ADDRESS
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         delegation03 => [
             qw(
               REFERRAL_SIZE_TOO_LARGE
               REFERRAL_SIZE_OK
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         delegation04 => [
@@ -82,12 +98,18 @@ sub metadata {
               IPV4_DISABLED
               IPV6_DISABLED
               ARE_AUTHORITATIVE
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         delegation05 => [
             qw(
-              NS_RR_IS_CNAME
-              NS_RR_NO_CNAME
+              NO_NS_CNAME
+              NO_RESPONSE    
+              NS_IS_CNAME
+              UNEXPECTED_RCODE
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         delegation06 => [
@@ -96,6 +118,8 @@ sub metadata {
               SOA_EXISTS
               IPV4_DISABLED
               IPV6_DISABLED
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         delegation07 => [
@@ -104,65 +128,204 @@ sub metadata {
               EXTRA_NAME_CHILD
               TOTAL_NAME_MISMATCH
               NAMES_MATCH
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
     };
 } ## end sub metadata
 
-sub translation {
-    return {
-        ARE_AUTHORITATIVE    => "All these nameservers are confirmed to be authoritative : {nsset}.",
-        CHILD_DISTINCT_NS_IP => "All the IP addresses used by the nameservers in child are unique.",
-        CHILD_NS_SAME_IP     => "IP {address} in child refers to multiple nameservers ({nss}).",
-        DEL_DISTINCT_NS_IP   => "All the IP addresses used by the nameservers in parent are unique.",
-        DEL_NS_SAME_IP       => "IP {address} in parent refers to multiple nameservers ({nss}).",
-        DISTINCT_IP_ADDRESS  => "All the IP addresses used by the nameservers are unique",
-        ENOUGH_IPV4_NS_CHILD => "Child lists enough ({count}) nameservers that resolve to IPv4 addresses ({addrs}). "
-          . "Lower limit set to {minimum}.",
-        ENOUGH_IPV4_NS_DEL => "Delegation lists enough ({count}) nameservers that resolve to IPv4 addresses ({addrs}). "
-          . "Lower limit set to {minimum}.",
-        ENOUGH_IPV6_NS_CHILD => "Child lists enough ({count}) nameservers that resolve to IPv6 addresses ({addrs}). "
-          . "Lower limit set to {minimum}.",
-        ENOUGH_IPV6_NS_DEL => "Delegation lists enough ({count}) nameservers that resolve to IPv6 addresses ({addrs}). "
-          . "Lower limit set to {minimum}.",
-        ENOUGH_NS_CHILD          => "Child lists enough ({count}) nameservers ({ns}). Lower limit set to {minimum}.",
-        ENOUGH_NS_DEL            => "Parent lists enough ({count}) nameservers ({glue}). Lower limit set to {minimum}.",
-        EXTRA_NAME_CHILD         => "Child has nameserver(s) not listed at parent ({extra}).",
-        EXTRA_NAME_PARENT        => "Parent has nameserver(s) not listed at the child ({extra}).",
-        IPV4_DISABLED            => 'IPv4 is disabled, not sending "{rrtype}" query to {ns}/{address}.',
-        IPV6_DISABLED            => 'IPv6 is disabled, not sending "{rrtype}" query to {ns}/{address}.',
-        IS_NOT_AUTHORITATIVE     => "Nameserver {ns} response is not authoritative on {proto} port 53.",
-        NAMES_MATCH              => "All of the nameserver names are listed both at parent and child.",
-        NOT_ENOUGH_IPV4_NS_CHILD => "Child does not list enough ({count}) nameservers that "
-          . "resolve to IPv4 addresses ({addrs}). Lower limit set to {minimum}.",
-        NOT_ENOUGH_IPV4_NS_DEL => "Delegation does not list enough ({count}) nameservers that "
-          . "resolve to IPv4 addresses ({addrs}). Lower limit set to {minimum}.",
-        NOT_ENOUGH_IPV6_NS_CHILD => "Child does not list enough ({count}) nameservers that "
-          . "resolve to IPv6 addresses ({addrs}). Lower limit set to {minimum}.",
-        NOT_ENOUGH_IPV6_NS_DEL => "Delegation does not list enough ({count}) nameservers that "
-          . "resolve to IPv6 addresses ({addrs}). Lower limit set to {minimum}.",
-        NOT_ENOUGH_NS_CHILD => "Child does not list enough ({count}) nameservers ({ns}). Lower limit set to {minimum}.",
-        NOT_ENOUGH_NS_DEL   => "Parent does not list enough ({count}) nameservers ({glue}). "
-          . "Lower limit set to {minimum}.",
-        NO_IPV4_NS_CHILD => "Child lists no nameserver that resolves to an IPv4 address. If any were present, "
-          . "the minimum allowed would be {minimum}.",
-        NO_IPV4_NS_DEL => "Delegation lists no nameserver that resolves to an IPv4 address. If any were present, "
-          . "the minimum allowed would be {minimum}.",
-        NO_IPV6_NS_CHILD => "Child lists no nameserver that resolves to an IPv6 address. If any were present, "
-          . "the minimum allowed would be {minimum}.",
-        NO_IPV6_NS_DEL => "Delegation lists no nameserver that resolves to an IPv6 address. If any were present, "
-          . "the minimum allowed would be {minimum}.",
-        NS_RR_IS_CNAME          => "Nameserver {ns} {address_type} RR point to CNAME.",
-        NS_RR_NO_CNAME          => "No nameserver point to CNAME alias.",
-        REFERRAL_SIZE_TOO_LARGE => "The smallest possible legal referral packet is larger than 512 octets "
-          . "(it is {size}).",
-        REFERRAL_SIZE_OK    => "The smallest possible legal referral packet is smaller than 513 octets (it is {size}).",
-        SAME_IP_ADDRESS     => "IP {address} refers to multiple nameservers ({nss}).",
-        SOA_EXISTS          => "All the nameservers have SOA record.",
-        SOA_NOT_EXISTS      => "A SOA query NOERROR response from {ns} was received empty.",
-        TOTAL_NAME_MISMATCH => "None of the nameservers listed at the parent are listed at the child.",
-    };
-} ## end sub translation
+Readonly my %TAG_DESCRIPTIONS => (
+    ARE_AUTHORITATIVE => sub {
+        __x    # DELEGATION:ARE_AUTHORITATIVE
+          "All these nameservers are confirmed to be authoritative : {nsname_list}.", @_;
+    },
+    CHILD_DISTINCT_NS_IP => sub {
+        __x    # DELEGATION:CHILD_DISTINCT_NS_IP
+          "All the IP addresses used by the nameservers in child are unique.", @_;
+    },
+    CHILD_NS_SAME_IP => sub {
+        __x    # DELEGATION:CHILD_NS_SAME_IP
+          "IP {ns_ip} in child refers to multiple nameservers ({nsname_list}).", @_;
+    },
+    DEL_DISTINCT_NS_IP => sub {
+        __x    # DELEGATION:DEL_DISTINCT_NS_IP
+          "All the IP addresses used by the nameservers in parent are unique.", @_;
+    },
+    DEL_NS_SAME_IP => sub {
+        __x    # DELEGATION:DEL_NS_SAME_IP
+          "IP {ns_ip} in parent refers to multiple nameservers ({nsname_list}).", @_;
+    },
+    DISTINCT_IP_ADDRESS => sub {
+        __x    # DELEGATION:DISTINCT_IP_ADDRESS
+          "All the IP addresses used by the nameservers are unique.", @_;
+    },
+    ENOUGH_IPV4_NS_CHILD => sub {
+        __x    # DELEGATION:ENOUGH_IPV4_NS_CHILD
+          "Child lists enough ({count}) nameservers ({nsname_list}) "
+          . "that resolve to IPv4 addresses ({ns_ip_list}). Lower limit set to {minimum}.",
+          @_;
+    },
+    ENOUGH_IPV4_NS_DEL => sub {
+        __x    # DELEGATION:ENOUGH_IPV4_NS_DEL
+          "Delegation lists enough ({count}) nameservers ({nsname_list}) "
+          . "that resolve to IPv4 addresses ({ns_ip_list}). Lower limit set to {minimum}.",
+          @_;
+    },
+    ENOUGH_IPV6_NS_CHILD => sub {
+        __x    # DELEGATION:ENOUGH_IPV6_NS_CHILD
+          "Child lists enough ({count}) nameservers ({nsname_list}) "
+          . "that resolve to IPv6 addresses ({ns_ip_list}). Lower limit set to {minimum}.",
+          @_;
+    },
+    ENOUGH_IPV6_NS_DEL => sub {
+        __x    # DELEGATION:ENOUGH_IPV6_NS_DEL
+          "Delegation lists enough ({count}) nameservers ({nsname_list}) "
+          . "that resolve to IPv6 addresses ({ns_ip_list}). Lower limit set to {minimum}.",
+          @_;
+    },
+    ENOUGH_NS_CHILD => sub {
+        __x    # DELEGATION:ENOUGH_NS_CHILD
+          "Child lists enough ({count}) nameservers ({nsname_list}). Lower limit set to {minimum}.", @_;
+    },
+    ENOUGH_NS_DEL => sub {
+        __x    # DELEGATION:ENOUGH_NS_DEL
+          "Parent lists enough ({count}) nameservers ({nsname_list}). Lower limit set to {minimum}.", @_;
+    },
+    EXTRA_NAME_CHILD => sub {
+        __x    # DELEGATION:EXTRA_NAME_CHILD
+          "Child has nameserver(s) not listed at parent ({extra}).", @_;
+    },
+    EXTRA_NAME_PARENT => sub {
+        __x    # DELEGATION:EXTRA_NAME_PARENT
+          "Parent has nameserver(s) not listed at the child ({extra}).", @_;
+    },
+    IPV4_DISABLED => sub {
+        __x    # DELEGATION:IPV4_DISABLED
+          'IPv4 is disabled, not sending "{rrtype}" query to {ns}.', @_;
+    },
+    IPV6_DISABLED => sub {
+        __x    # DELEGATION:IPV6_DISABLED
+          'IPv6 is disabled, not sending "{rrtype}" query to {ns}.', @_;
+    },
+    IS_NOT_AUTHORITATIVE => sub {
+        __x    # DELEGATION:IS_NOT_AUTHORITATIVE
+          "Nameserver {ns} response is not authoritative on {proto} port 53.", @_;
+    },
+    NAMES_MATCH => sub {
+        __x    # DELEGATION:NAMES_MATCH
+          "All of the nameserver names are listed both at parent and child.", @_;
+    },
+    NO_RESPONSE => sub {
+        __x    # DELEGATION:NO_RESPONSE
+          "Nameserver {ns} did not respond.", @_;
+    },
+    NOT_ENOUGH_IPV4_NS_CHILD => sub {
+        __x    # DELEGATION:NOT_ENOUGH_IPV4_NS_CHILD
+          "Child does not list enough ({count}) nameservers ({nsname_list}) "
+          . "that resolve to IPv4 addresses ({ns_ip_list}). Lower limit set to {minimum}.",
+          @_;
+    },
+    NOT_ENOUGH_IPV4_NS_DEL => sub {
+        __x    # DELEGATION:NOT_ENOUGH_IPV4_NS_DEL
+          "Delegation does not list enough ({count}) nameservers ({nsname_list}) "
+          . "that resolve to IPv4 addresses ({ns_ip_list}). Lower limit set to {minimum}.",
+          @_;
+    },
+    NOT_ENOUGH_IPV6_NS_CHILD => sub {
+        __x    # DELEGATION:NOT_ENOUGH_IPV6_NS_CHILD
+          "Child does not list enough ({count}) nameservers ({nsname_list}) "
+          . "that resolve to IPv6 addresses ({ns_ip_list}). Lower limit set to {minimum}.",
+          @_;
+    },
+    NOT_ENOUGH_IPV6_NS_DEL => sub {
+        __x    # DELEGATION:NOT_ENOUGH_IPV6_NS_DEL
+          "Delegation does not list enough ({count}) nameservers ({nsname_list}) "
+          . "that resolve to IPv6 addresses ({ns_ip_list}). Lower limit set to {minimum}.",
+          @_;
+    },
+    NOT_ENOUGH_NS_CHILD => sub {
+        __x    # DELEGATION:NOT_ENOUGH_NS_CHILD
+          "Child does not list enough ({count}) nameservers ({nsname_list}). Lower limit set to {minimum}.", @_;
+    },
+    NOT_ENOUGH_NS_DEL => sub {
+        __x    # DELEGATION:NOT_ENOUGH_NS_DEL
+          "Parent does not list enough ({count}) nameservers ({nsname_list}). Lower limit set to {minimum}.", @_;
+    },
+    NO_IPV4_NS_CHILD => sub {
+        __x    # DELEGATION:NO_IPV4_NS_CHILD
+          "Child lists no nameserver that resolves to an IPv4 address. "
+          . "If any were present, the minimum allowed would be {minimum}.",
+          @_;
+    },
+    NO_IPV4_NS_DEL => sub {
+        __x    # DELEGATION:NO_IPV4_NS_DEL
+          "Delegation lists no nameserver that resolves to an IPv4 address. "
+          . "If any were present, the minimum allowed would be {minimum}.",
+          @_;
+    },
+    NO_IPV6_NS_CHILD => sub {
+        __x    # DELEGATION:NO_IPV6_NS_CHILD
+          "Child lists no nameserver that resolves to an IPv6 address. "
+          . "If any were present, the minimum allowed would be {minimum}.",
+          @_;
+    },
+    NO_IPV6_NS_DEL => sub {
+        __x    # DELEGATION:NO_IPV6_NS_DEL
+          "Delegation lists no nameserver that resolves to an IPv6 address. "
+          . "If any were present, the minimum allowed would be {minimum}.",
+          @_;
+    },
+    NS_IS_CNAME => sub {
+        __x    # DELEGATION:NS_IS_CNAME
+          "Nameserver {nsname} RR point to CNAME.", @_;
+    },
+    NO_NS_CNAME => sub {
+        __x    # DELEGATION:NO_NS_CNAME
+          "No nameserver points to CNAME alias.", @_;
+    },
+    REFERRAL_SIZE_TOO_LARGE => sub {
+        __x    # DELEGATION:REFERRAL_SIZE_TOO_LARGE
+          "The smallest possible legal referral packet is larger than 512 octets (it is {size}).", @_;
+    },
+    REFERRAL_SIZE_OK => sub {
+        __x    # DELEGATION:REFERRAL_SIZE_OK
+          "The smallest possible legal referral packet is smaller than 513 octets (it is {size}).", @_;
+    },
+    SAME_IP_ADDRESS => sub {
+        __x    # DELEGATION:SAME_IP_ADDRESS
+          "IP {ns_ip} refers to multiple nameservers ({nsname_list}).", @_;
+    },
+    SOA_EXISTS => sub {
+        __x    # DELEGATION:SOA_EXISTS
+          "All the nameservers have SOA record.", @_;
+    },
+    SOA_NOT_EXISTS => sub {
+        __x    # DELEGATION:SOA_NOT_EXISTS
+          "Empty NOERROR response to SOA query was received from {ns}.", @_;
+    },
+    TEST_CASE_END => sub {
+        __x    # DELEGATION:TEST_CASE_END
+          'TEST_CASE_END {testcase}.', @_;
+    },
+    TEST_CASE_START => sub {
+        __x    # DELEGATION:TEST_CASE_START
+          'TEST_CASE_START {testcase}.', @_;
+    },
+    TOTAL_NAME_MISMATCH => sub {
+        __x    # DELEGATION:TOTAL_NAME_MISMATCH
+          "None of the nameservers listed at the parent are listed at the child.", @_;
+    },
+    UNEXPECTED_RCODE => sub {
+        __x    # DELEGATION:UNEXPECTED_RCODE
+          'Nameserver {ns} answered query with an unexpected rcode ({rcode}).', @_;
+    },
+
+);
+
+sub tag_descriptions {
+    return \%TAG_DESCRIPTIONS;
+}
 
 sub version {
     return "$Zonemaster::Engine::Test::Delegation::VERSION";
@@ -174,14 +337,14 @@ sub version {
 
 sub delegation01 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     # Determine delegation NS names
     my @del_nsnames = map { $_->string } @{ Zonemaster::Engine::TestMethods->method2( $zone ) };
     my $del_nsnames_args = {
-        count   => scalar( @del_nsnames ),
-        minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-        glue    => join( q{;}, sort @del_nsnames ),
+        count       => scalar( @del_nsnames ),
+        minimum     => $MINIMUM_NUMBER_OF_NAMESERVERS,
+        nsname_list => join( q{;}, sort @del_nsnames ),
     };
 
     # Check delegation NS names
@@ -189,16 +352,15 @@ sub delegation01 {
         push @results, info( ENOUGH_NS_DEL => $del_nsnames_args );
     }
     else {
-        push @results,
-          info( NOT_ENOUGH_NS_DEL => $del_nsnames_args );
+        push @results, info( NOT_ENOUGH_NS_DEL => $del_nsnames_args );
     }
 
     # Determine child NS names
     my @child_nsnames = map { $_->string } @{ Zonemaster::Engine::TestMethods->method3( $zone ) };
     my $child_nsnames_args = {
-        count   => scalar( @child_nsnames ),
-        minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-        ns      => join( q{;}, sort @child_nsnames ),
+        count       => scalar( @child_nsnames ),
+        minimum     => $MINIMUM_NUMBER_OF_NAMESERVERS,
+        nsname_list => join( q{;}, sort @child_nsnames ),
     };
 
     # Check child NS names
@@ -206,8 +368,7 @@ sub delegation01 {
         push @results, info( ENOUGH_NS_CHILD => $child_nsnames_args );
     }
     else {
-        push @results,
-          info( NOT_ENOUGH_NS_CHILD => $child_nsnames_args );
+        push @results, info( NOT_ENOUGH_NS_CHILD => $child_nsnames_args );
     }
 
     # Determine child NS names with addresses
@@ -215,19 +376,19 @@ sub delegation01 {
     my @child_ns_ipv4 = uniq map { $_->name->string } grep { $_->address->version == 4 } @child_ns;
     my @child_ns_ipv6 = uniq map { $_->name->string } grep { $_->address->version == 6 } @child_ns;
     my @child_ns_ipv4_addrs = uniq map { $_->address->ip } grep { $_->address->version == 4 } @child_ns;
-    my @child_ns_ipv6_addrs = uniq map { $_->address->short } grep { $_->address->version == 4 } @child_ns;
+    my @child_ns_ipv6_addrs = uniq map { $_->address->short } grep { $_->address->version == 6 } @child_ns;
 
     my $child_ns_ipv4_args = {
-        count   => scalar( @child_ns_ipv4 ),
-        minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-        ns      => join( q{;}, sort @child_ns_ipv4 ),
-	addrs   => join( q{;}, sort @child_ns_ipv4_addrs ),
+        count       => scalar( @child_ns_ipv4 ),
+        minimum     => $MINIMUM_NUMBER_OF_NAMESERVERS,
+        nsname_list => join( q{;}, sort @child_ns_ipv4 ),
+        ns_ip_list  => join( q{;}, sort @child_ns_ipv4_addrs ),
     };
     my $child_ns_ipv6_args = {
-        count   => scalar( @child_ns_ipv6 ),
-        minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-        ns      => join( q{;}, sort @child_ns_ipv6 ),
-	addrs   => join( q{;}, sort @child_ns_ipv6_addrs ),
+        count       => scalar( @child_ns_ipv6 ),
+        minimum     => $MINIMUM_NUMBER_OF_NAMESERVERS,
+        nsname_list => join( q{;}, sort @child_ns_ipv6 ),
+        ns_ip_list  => join( q{;}, sort @child_ns_ipv6_addrs ),
     };
 
     if ( scalar( @child_ns_ipv4 ) >= $MINIMUM_NUMBER_OF_NAMESERVERS ) {
@@ -258,16 +419,16 @@ sub delegation01 {
     my @del_ns_ipv6_addrs = uniq map { $_->address->short } grep { $_->address->version == 6 } @del_ns;
 
     my $del_ns_ipv4_args = {
-        count   => scalar( @del_ns_ipv4 ),
-        minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-        ns      => join( q{;}, sort @del_ns_ipv4 ),
-	addrs   => join( q{;}, sort @del_ns_ipv4_addrs ),
+        count       => scalar( @del_ns_ipv4 ),
+        minimum     => $MINIMUM_NUMBER_OF_NAMESERVERS,
+        nsname_list => join( q{;}, sort @del_ns_ipv4 ),
+        ns_ip_list  => join( q{;}, sort @del_ns_ipv4_addrs ),
     };
     my $del_ns_ipv6_args = {
-        count   => scalar( @del_ns_ipv6 ),
-        minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-        ns      => join( q{;}, sort @del_ns_ipv6 ),
-	addrs   => join( q{;}, sort @del_ns_ipv6_addrs ),
+        count       => scalar( @del_ns_ipv6 ),
+        minimum     => $MINIMUM_NUMBER_OF_NAMESERVERS,
+        nsname_list => join( q{;}, sort @del_ns_ipv6 ),
+        ns_ip_list  => join( q{;}, sort @del_ns_ipv6_addrs ),
     };
 
     if ( scalar( @del_ns_ipv4 ) >= $MINIMUM_NUMBER_OF_NAMESERVERS ) {
@@ -290,14 +451,14 @@ sub delegation01 {
         push @results, info( NO_IPV6_NS_DEL => $del_ns_ipv6_args );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub delegation01
 
 sub _find_dup_ns {
-    my %args = @_;
+    my %args          = @_;
     my $duplicate_tag = $args{duplicate_tag};
-    my $distinct_tag = $args{distinct_tag};
-    my @nss = @{ $args{nss} };
+    my $distinct_tag  = $args{distinct_tag};
+    my @nss           = @{ $args{ns_list} };
 
     my %nsnames_and_ip;
     my %ips;
@@ -317,8 +478,8 @@ sub _find_dup_ns {
             push @results,
               info(
                 $duplicate_tag => {
-                    nss     => join( q{;}, @{ $ips{$local_ip} } ),
-                    address => $local_ip,
+                    nsname_list => join( q{;}, @{ $ips{$local_ip} } ),
+                    ns_ip       => $local_ip,
                 }
               );
         }
@@ -333,7 +494,7 @@ sub _find_dup_ns {
 
 sub delegation02 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my @nss_del   = @{ Zonemaster::Engine::TestMethods->method4( $zone ) };
     my @nss_child = @{ Zonemaster::Engine::TestMethods->method5( $zone ) };
@@ -342,29 +503,29 @@ sub delegation02 {
       _find_dup_ns(
         duplicate_tag => 'DEL_NS_SAME_IP',
         distinct_tag  => 'DEL_DISTINCT_NS_IP',
-        nss           => [@nss_del],
+        ns_list       => [@nss_del],
       );
 
     push @results,
       _find_dup_ns(
         duplicate_tag => 'CHILD_NS_SAME_IP',
         distinct_tag  => 'CHILD_DISTINCT_NS_IP',
-        nss           => [@nss_child],
+        ns_list       => [@nss_child],
       );
 
     push @results,
       _find_dup_ns(
         duplicate_tag => 'SAME_IP_ADDRESS',
         distinct_tag  => 'DISTINCT_IP_ADDRESS',
-        nss           => [ @nss_del, @nss_child ],
+        ns_list       => [ @nss_del, @nss_child ],
       );
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub delegation02
 
 sub delegation03 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my $long_name = _max_length_name_for( $zone->name );
     my @nsnames   = map { $_->string } @{ Zonemaster::Engine::TestMethods->method2( $zone ) };
@@ -380,14 +541,14 @@ sub delegation03 {
     }
 
     # If @nss_v4 is non-empty and all of its elements are in bailiwick of parent
-    if ( @nss_v4 && not grep { not $parent->name->is_in_bailiwick( $_->name ) } @nss_v4 ) {
+    if ( @nss_v4 and not grep { not $parent->name->is_in_bailiwick( $_->name ) } @nss_v4 ) {
         my $ns = $nss_v4[0];
         my $rr = Zonemaster::LDNS::RR->new( sprintf( q{%s IN A %s}, $ns->name, $ns->address->short ) );
         $p->unique_push( q{additional}, $rr );
     }
 
     # If @nss_v6 is non-empty and all of its elements are in bailiwick of parent
-    if ( @nss_v6 && not grep { not $parent->name->is_in_bailiwick( $_->name ) } @nss_v6 ) {
+    if ( @nss_v6 and not grep { not $parent->name->is_in_bailiwick( $_->name ) } @nss_v6 ) {
         my $ns = $nss_v6[0];
         my $rr = Zonemaster::LDNS::RR->new( sprintf( q{%s IN AAAA %s}, $ns->name, $ns->address->short ) );
         $p->unique_push( q{additional}, $rr );
@@ -411,12 +572,12 @@ sub delegation03 {
           );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub delegation03
 
 sub delegation04 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my %nsnames;
     my @authoritatives;
     my $query_type = q{SOA};
@@ -429,9 +590,8 @@ sub delegation04 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -441,9 +601,8 @@ sub delegation04 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -458,7 +617,7 @@ sub delegation04 {
                     push @results,
                       info(
                         IS_NOT_AUTHORITATIVE => {
-                            ns    => $local_ns->name->string,
+                            ns    => $local_ns->string,
                             proto => $usevc ? q{TCP} : q{UDP},
                         }
                       );
@@ -477,64 +636,91 @@ sub delegation04 {
                scalar @{ Zonemaster::Engine::TestMethods->method4( $zone ) }
             or scalar @{ Zonemaster::Engine::TestMethods->method5( $zone ) }
         )
-        and not scalar @results
+        and not grep { $_->tag ne q{TEST_CASE_START} } @results
         and scalar @authoritatives
       )
     {
         push @results,
           info(
             ARE_AUTHORITATIVE => {
-                nsset => join( q{,}, uniq sort @authoritatives ),
+                nsname_list => join( q{;}, uniq sort @authoritatives ),
             }
           );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub delegation04
 
 sub delegation05 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
-    my @nsnames = uniq map { $_->string } @{ Zonemaster::Engine::TestMethods->method2( $zone ) },
-      @{ Zonemaster::Engine::TestMethods->method3( $zone ) };
+    my @nsnames = @{ Zonemaster::Engine::TestMethods->method2and3( $zone ) };
 
-    foreach my $local_nsname ( @nsnames ) {
+    foreach my $local_nsname ( @nsnames )  {
 
-        foreach my $address_type ( q{A}, q{AAAA} ) {
-            my $p = $zone->query_one( $local_nsname, $address_type );
-            if ( $p ) {
-                if ( $p->has_rrs_of_type_for_name( q{CNAME}, $zone->name ) ) {
-                    push @results,
-                      info(
-                        NS_RR_IS_CNAME => {
-                            ns           => $local_nsname,
-                            address_type => $address_type,
-                        }
-                      );
+        if ( $zone->name->is_in_bailiwick( $local_nsname ) ) {
+            my @nss_del   = @{ Zonemaster::Engine::TestMethods->method4( $zone ) };
+            my @nss_child = @{ Zonemaster::Engine::TestMethods->method5( $zone ) };
+            my %nss       = map { $_->name->string . '/' . $_->address->short => $_ } @nss_del, @nss_child;
+
+            for my $key ( sort keys %nss ) {
+                my $ns = $nss{$key};
+                my $ns_args = {
+                    ns     => $ns->string,
+                    rrtype => q{A},
+                };
+
+                if ( not Zonemaster::Engine::Profile->effective->get(q{net.ipv6}) and $ns->address->version == $IP_VERSION_6 ) {
+                    push @results, info( IPV6_DISABLED => $ns_args );
+                    next;
+                }
+
+                if ( not Zonemaster::Engine::Profile->effective->get(q{net.ipv4}) and $ns->address->version == $IP_VERSION_4 ) {
+                    push @results, info( IPV4_DISABLED => $ns_args );
+                    next;
+                }
+
+                my $p = $ns->query( $local_nsname, q{A}, { recurse => 0 } );
+                if ( not $p ) {
+                    push @results, info( NO_RESPONSE => $ns_args );
+                    next;
+                }
+                elsif ($p->rcode ne q{NOERROR} ) {
+                    $ns_args->{rcode} = $p->rcode;
+                    push @results, info( UNEXPECTED_RCODE => $ns_args );
+                    next;
+                }
+                elsif ( scalar $p->get_records( q{CNAME}, q{answer} ) > 0 ) {
+                    push @results, info( NS_IS_CNAME => { nsname => $local_nsname } );
+                    next;
+                }
+                elsif ($p->is_redirect) {
+                    my $p = $ns->query( $local_nsname, q{A}, { recurse => 1 } );
+                    if ( defined $p and scalar $p->get_records( q{CNAME}, q{answer} ) > 0 ) {
+                        push @results, info( NS_IS_CNAME => { nsname => $local_nsname } );
+                    }
                 }
             }
         }
-
+        else {
+            my $p = Zonemaster::Engine::Recursor->recurse( $local_nsname, q{A} );
+            if ( defined $p and scalar $p->get_records( q{CNAME}, q{answer} ) > 0 ) {
+                push @results, info( NS_IS_CNAME => { nsname => $local_nsname } );
+            }
+        }
     }
 
-    if (
-        (
-               scalar @{ Zonemaster::Engine::TestMethods->method2( $zone ) }
-            or scalar @{ Zonemaster::Engine::TestMethods->method3( $zone ) }
-        )
-        and not scalar @results
-      )
-    {
-        push @results, info( NS_RR_NO_CNAME => {} );
+    if ( not grep { $_->tag eq q{NS_IS_CNAME} } @results ) {
+        push @results, info( NO_NS_CNAME => {} );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub delegation05
 
 sub delegation06 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my %nsnames;
     my $query_type = q{SOA};
 
@@ -546,9 +732,8 @@ sub delegation06 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -558,9 +743,8 @@ sub delegation06 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns      => $local_ns->name->string,
-                    address => $local_ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $local_ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -571,12 +755,7 @@ sub delegation06 {
         my $p = $local_ns->query( $zone->name, $query_type );
         if ( $p and $p->rcode eq q{NOERROR} ) {
             if ( not $p->get_records( $query_type, q{answer} ) ) {
-                push @results,
-                  info(
-                    SOA_NOT_EXISTS => {
-                        ns => $local_ns->name->string,
-                    }
-                  );
+                push @results, info( SOA_NOT_EXISTS => { nsname => $local_ns->string } );
             }
         }
 
@@ -588,18 +767,18 @@ sub delegation06 {
                scalar @{ Zonemaster::Engine::TestMethods->method4( $zone ) }
             or scalar @{ Zonemaster::Engine::TestMethods->method5( $zone ) }
         )
-        and not scalar @results
+        and not grep { $_->tag ne q{TEST_CASE_START} } @results
       )
     {
         push @results, info( SOA_EXISTS => {} );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub delegation06
 
 sub delegation07 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my %names;
     foreach my $name ( @{ Zonemaster::Engine::TestMethods->method2( $zone ) } ) {
@@ -650,7 +829,7 @@ sub delegation07 {
           );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub delegation07
 
 ###
@@ -692,9 +871,9 @@ Zonemaster::Engine::Test::Delegation - Tests regarding delegation details
 
 Runs the default set of tests and returns a list of log entries made by the tests.
 
-=item translation()
+=item tag_descriptions()
 
-Returns a refernce to a hash with translation data. Used by the builtin translation system.
+Returns a refernce to a hash with translation functions. Used by the builtin translation system.
 
 =item metadata()
 
