@@ -1,26 +1,28 @@
 package Zonemaster::Engine::Test::Syntax;
 
-use version; our $VERSION = version->declare("v1.0.5");
+use 5.014002;
 
 use strict;
 use warnings;
 
-use 5.014002;
+use version; our $VERSION = version->declare( "v1.0.9" );
 
 use Zonemaster::Engine;
-use Zonemaster::Engine::Util;
-use Zonemaster::Engine::Recursor;
-use Zonemaster::Engine::DNSName;
-use Zonemaster::Engine::TestMethods;
-use Zonemaster::Engine::Constants qw[:name];
-use Zonemaster::LDNS;
-use Zonemaster::Engine::Packet;
 
 use Carp;
-
+use Email::Valid;
 use List::MoreUtils qw[uniq none any];
-use Mail::RFC822::Address qw[valid];
+use Locale::TextDomain qw[Zonemaster-Engine];
+use Readonly;
 use Time::Local;
+use Zonemaster::Engine::Profile;
+use Zonemaster::Engine::Constants qw[:name :ip];
+use Zonemaster::Engine::DNSName;
+use Zonemaster::Engine::Packet;
+use Zonemaster::Engine::Recursor;
+use Zonemaster::Engine::TestMethods;
+use Zonemaster::Engine::Util;
+use Zonemaster::LDNS;
 
 ###
 ### Entry points
@@ -39,7 +41,8 @@ sub all {
         foreach my $local_nsname ( uniq map { $_->string } @{ Zonemaster::Engine::TestMethods->method2( $zone ) },
             @{ Zonemaster::Engine::TestMethods->method3( $zone ) } )
         {
-            push @results, $class->syntax04( $local_nsname ) if Zonemaster::Engine::Util::should_run_test( q{syntax04} );
+            push @results, $class->syntax04( $local_nsname )
+              if Zonemaster::Engine::Util::should_run_test( q{syntax04} );
         }
 
         push @results, $class->syntax05( $zone ) if Zonemaster::Engine::Util::should_run_test( q{syntax05} );
@@ -68,6 +71,8 @@ sub metadata {
             qw(
               ONLY_ALLOWED_CHARS
               NON_ALLOWED_CHARS
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         syntax02 => [
@@ -75,12 +80,16 @@ sub metadata {
               INITIAL_HYPHEN
               TERMINAL_HYPHEN
               NO_ENDING_HYPHENS
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         syntax03 => [
             qw(
               DISCOURAGED_DOUBLE_DASH
               NO_DOUBLE_DASH
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         syntax04 => [
@@ -89,6 +98,8 @@ sub metadata {
               NAMESERVER_NON_ALLOWED_CHARS
               NAMESERVER_NUMERIC_TLD
               NAMESERVER_SYNTAX_OK
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         syntax05 => [
@@ -96,6 +107,8 @@ sub metadata {
               RNAME_MISUSED_AT_SIGN
               RNAME_NO_AT_SIGN
               NO_RESPONSE_SOA_QUERY
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         syntax06 => [
@@ -105,6 +118,8 @@ sub metadata {
               RNAME_MAIL_DOMAIN_INVALID
               RNAME_RFC822_INVALID
               RNAME_RFC822_VALID
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         syntax07 => [
@@ -114,6 +129,8 @@ sub metadata {
               MNAME_NUMERIC_TLD
               MNAME_SYNTAX_OK
               NO_RESPONSE_SOA_QUERY
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         syntax08 => [
@@ -123,48 +140,153 @@ sub metadata {
               MX_NUMERIC_TLD
               MX_SYNTAX_OK
               NO_RESPONSE_MX_QUERY
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
     };
 } ## end sub metadata
 
-sub translation {
-    return {
-        DISCOURAGED_DOUBLE_DASH => 'Domain name ({name}) has a label ({label}) with a double hyphen (\'--\') '
+Readonly my %TAG_DESCRIPTIONS => (
+    DISCOURAGED_DOUBLE_DASH => sub {
+        __x    # SYNTAX:DISCOURAGED_DOUBLE_DASH
+          'Domain name ({name}) has a label ({label}) with a double hyphen (\'--\') '
           . 'in position 3 and 4 (with a prefix which is not \'xn--\').',
-        INITIAL_HYPHEN                => 'Domain name ({name}) has a label ({label}) starting with an hyphen (\'-\').',
-        MNAME_DISCOURAGED_DOUBLE_DASH => 'SOA MNAME ({name}) has a label ({label}) with a double hyphen (\'--\') '
+          @_;
+    },
+    INITIAL_HYPHEN => sub {
+        __x    # SYNTAX:INITIAL_HYPHEN
+          'Domain name ({name}) has a label ({label}) starting with an hyphen (\'-\').', @_;
+    },
+    IPV4_DISABLED => sub {
+        __x    # SYNTAX:IPV4_DISABLED
+          'IPv4 is disabled, not sending "{rrtype}" query to {ns}.', @_;
+    },
+    IPV6_DISABLED => sub {
+        __x    # SYNTAX:IPV6_DISABLED
+          'IPv6 is disabled, not sending "{rrtype}" query to {ns}.', @_;
+    },
+    MNAME_DISCOURAGED_DOUBLE_DASH => sub {
+        __x    # SYNTAX:MNAME_DISCOURAGED_DOUBLE_DASH
+          'SOA MNAME ({name}) has a label ({label}) with a double hyphen (\'--\') '
           . 'in position 3 and 4 (with a prefix which is not \'xn--\').',
-        MNAME_NON_ALLOWED_CHARS    => 'Found illegal characters in SOA MNAME ({name}).',
-        MNAME_NUMERIC_TLD          => 'SOA MNAME ({name}) within a \'numeric only\' TLD ({tld}).',
-        MNAME_SYNTAX_OK            => 'SOA MNAME ({name}) syntax is valid.',
-        MX_DISCOURAGED_DOUBLE_DASH => 'Domain name MX ({name}) has a label ({label}) with a double hyphen (\'--\') '
+          @_;
+    },
+    MNAME_NON_ALLOWED_CHARS => sub {
+        __x    # SYNTAX:MNAME_NON_ALLOWED_CHARS
+          'Found illegal characters in SOA MNAME ({name}).', @_;
+    },
+    MNAME_NUMERIC_TLD => sub {
+        __x    # SYNTAX:MNAME_NUMERIC_TLD
+          'SOA MNAME ({name}) within a \'numeric only\' TLD ({tld}).', @_;
+    },
+    MNAME_SYNTAX_OK => sub {
+        __x    # SYNTAX:MNAME_SYNTAX_OK
+          'SOA MNAME ({name}) syntax is valid.', @_;
+    },
+    MX_DISCOURAGED_DOUBLE_DASH => sub {
+        __x    # SYNTAX:MX_DISCOURAGED_DOUBLE_DASH
+          'Domain name MX ({name}) has a label ({label}) with a double hyphen (\'--\') '
           . 'in position 3 and 4 (with a prefix which is not \'xn--\').',
-        MX_NON_ALLOWED_CHARS               => 'Found illegal characters in MX ({name}).',
-        MX_NUMERIC_TLD                     => 'Domain name MX ({name}) within a \'numeric only\' TLD ({tld}).',
-        MX_SYNTAX_OK                       => 'Domain name MX ({name}) syntax is valid.',
-        NAMESERVER_DISCOURAGED_DOUBLE_DASH => 'Nameserver ({name}) has a label ({label}) '
-          . 'with a double hyphen (\'--\') in position 3 and 4 (with a prefix which is not \'xn--\').',
-        NAMESERVER_NON_ALLOWED_CHARS => 'Found illegal characters in the nameserver ({name}).',
-        NAMESERVER_NUMERIC_TLD       => 'Nameserver ({name}) within a \'numeric only\' TLD ({tld}).',
-        NAMESERVER_SYNTAX_OK         => 'Nameserver ({name}) syntax is valid.',
-        NON_ALLOWED_CHARS            => 'Found illegal characters in the domain name ({name}).',
-        NO_DOUBLE_DASH               => 'Domain name ({name}) has no label with a double hyphen (\'--\') '
+          @_;
+    },
+    MX_NON_ALLOWED_CHARS => sub {
+        __x    # SYNTAX:MX_NON_ALLOWED_CHARS
+          'Found illegal characters in MX ({name}).', @_;
+    },
+    MX_NUMERIC_TLD => sub {
+        __x    # SYNTAX:MX_NUMERIC_TLD
+          'Domain name MX ({name}) within a \'numeric only\' TLD ({tld}).', @_;
+    },
+    MX_SYNTAX_OK => sub {
+        __x    # SYNTAX:MX_SYNTAX_OK
+          'Domain name MX ({name}) syntax is valid.', @_;
+    },
+    NAMESERVER_DISCOURAGED_DOUBLE_DASH => sub {
+        __x    # SYNTAX:NAMESERVER_DISCOURAGED_DOUBLE_DASH
+          'Nameserver ({name}) has a label ({label}) with a double hyphen (\'--\') '
           . 'in position 3 and 4 (with a prefix which is not \'xn--\').',
-        NO_ENDING_HYPHENS         => 'Both ends of all labels of the domain name ({name}) have no hyphens.',
-        NO_RESPONSE               => 'No response from {ns}/{address} asking for {dname}.',
-        NO_RESPONSE_MX_QUERY      => 'No response from nameserver(s) on MX queries.',
-        NO_RESPONSE_SOA_QUERY     => 'No response from nameserver(s) on SOA queries.',
-        ONLY_ALLOWED_CHARS        => 'No illegal characters in the domain name ({name}).',
-        RNAME_MAIL_DOMAIN_INVALID => 'The SOA RNAME mail domain ({domain}) cannot be resolved to a mail server '
-          . 'with an IP address.',
-        RNAME_MISUSED_AT_SIGN => 'There must be no misused \'@\' character in the SOA RNAME field ({rname}).',
-        RNAME_NO_AT_SIGN      => 'There is no misused \'@\' character in the SOA RNAME field ({rname}).',
-        RNAME_RFC822_INVALID  => 'There must be no illegal characters in the SOA RNAME field ({rname}).',
-        RNAME_RFC822_VALID    => 'The SOA RNAME field ({rname}) is compliant with RFC2822.',
-        TERMINAL_HYPHEN       => 'Domain name ({name}) has a label ({label}) ending with an hyphen (\'-\').',
-    };
-} ## end sub translation
+          @_;
+    },
+    NAMESERVER_NON_ALLOWED_CHARS => sub {
+        __x    # SYNTAX:NAMESERVER_NON_ALLOWED_CHARS
+          'Found illegal characters in the nameserver ({name}).', @_;
+    },
+    NAMESERVER_NUMERIC_TLD => sub {
+        __x    # SYNTAX:NAMESERVER_NUMERIC_TLD
+          'Nameserver ({name}) within a \'numeric only\' TLD ({tld}).', @_;
+    },
+    NAMESERVER_SYNTAX_OK => sub {
+        __x    # SYNTAX:NAMESERVER_SYNTAX_OK
+          'Nameserver ({name}) syntax is valid.', @_;
+    },
+    NON_ALLOWED_CHARS => sub {
+        __x    # SYNTAX:NON_ALLOWED_CHARS
+          'Found illegal characters in the domain name ({name}).', @_;
+    },
+    NO_DOUBLE_DASH => sub {
+        __x    # SYNTAX:NO_DOUBLE_DASH
+          'Domain name ({name}) has no label with a double hyphen (\'--\') '
+          . 'in position 3 and 4 (with a prefix which is not \'xn--\').',
+          @_;
+    },
+    NO_ENDING_HYPHENS => sub {
+        __x    # SYNTAX:NO_ENDING_HYPHENS
+          "Neither end of any label in the domain name ({name}) has a hyphen.", @_;
+    },
+    NO_RESPONSE => sub {
+        __x    # SYNTAX:NO_RESPONSE
+          'No response from {ns} asking for {dname}.', @_;
+    },
+    NO_RESPONSE_MX_QUERY => sub {
+        __x    # SYNTAX:NO_RESPONSE_MX_QUERY
+          'No response from nameserver(s) on MX queries.', @_;
+    },
+    NO_RESPONSE_SOA_QUERY => sub {
+        __x    # SYNTAX:NO_RESPONSE_SOA_QUERY
+          'No response from nameserver(s) on SOA queries.', @_;
+    },
+    ONLY_ALLOWED_CHARS => sub {
+        __x    # SYNTAX:ONLY_ALLOWED_CHARS
+          'No illegal characters in the domain name ({name}).', @_;
+    },
+    RNAME_MAIL_DOMAIN_INVALID => sub {
+        __x    # SYNTAX:RNAME_MAIL_DOMAIN_INVALID
+          'The SOA RNAME mail domain ({domain}) cannot be resolved to a mail server with an IP address.', @_;
+    },
+    RNAME_MISUSED_AT_SIGN => sub {
+        __x    # SYNTAX:RNAME_MISUSED_AT_SIGN
+          "Misused '\@' character found in SOA RNAME field ({rname}).", @_;
+    },
+    RNAME_NO_AT_SIGN => sub {
+        __x    # SYNTAX:RNAME_NO_AT_SIGN
+          'There is no misused \'@\' character in the SOA RNAME field ({rname}).', @_;
+    },
+    RNAME_RFC822_INVALID => sub {
+        __x    # SYNTAX:RNAME_RFC822_INVALID
+           "Illegal character(s) found in SOA RNAME field ({rname}).", @_;
+    },
+    RNAME_RFC822_VALID => sub {
+        __x    # SYNTAX:RNAME_RFC822_VALID
+          'The SOA RNAME field ({rname}) is compliant with RFC2822.', @_;
+    },
+    TERMINAL_HYPHEN => sub {
+        __x    # SYNTAX:TERMINAL_HYPHEN
+          "Domain name ({name}) has a label ({label}) ending with a hyphen ('-').", @_;
+    },
+    TEST_CASE_END => sub {
+        __x    # SYNTAX:TEST_CASE_END
+          'TEST_CASE_END {testcase}.', @_;
+    },
+    TEST_CASE_START => sub {
+        __x    # SYNTAX:TEST_CASE_START
+          'TEST_CASE_START {testcase}.', @_;
+    },
+);
+
+sub tag_descriptions {
+    return \%TAG_DESCRIPTIONS;
+}
 
 sub version {
     return "$Zonemaster::Engine::Test::Syntax::VERSION";
@@ -176,7 +298,7 @@ sub version {
 
 sub syntax01 {
     my ( $class, $item ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my $name = get_name( $item );
 
@@ -197,12 +319,12 @@ sub syntax01 {
           );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub syntax01
 
 sub syntax02 {
     my ( $class, $item ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my $name = get_name( $item );
 
@@ -227,7 +349,7 @@ sub syntax02 {
         }
     } ## end foreach my $local_label ( @...)
 
-    if ( scalar @{ $name->labels } and not scalar @results ) {
+    if ( scalar @{ $name->labels } and not grep { $_->tag ne q{TEST_CASE_START} } @results ) {
         push @results,
           info(
             NO_ENDING_HYPHENS => {
@@ -236,12 +358,12 @@ sub syntax02 {
           );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub syntax02
 
 sub syntax03 {
     my ( $class, $item ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my $name = get_name( $item );
 
@@ -257,7 +379,7 @@ sub syntax03 {
         }
     }
 
-    if ( scalar @{ $name->labels } and not scalar @results ) {
+    if ( scalar @{ $name->labels } and not grep { $_->tag ne q{TEST_CASE_START} } @results ) {
         push @results,
           info(
             NO_DOUBLE_DASH => {
@@ -266,23 +388,23 @@ sub syntax03 {
           );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub syntax03
 
 sub syntax04 {
     my ( $class, $item ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my $name = get_name( $item );
 
     push @results, check_name_syntax( q{NAMESERVER}, $name );
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 }
 
 sub syntax05 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my $p = $zone->query_one( $zone->name, q{SOA} );
 
@@ -310,12 +432,12 @@ sub syntax05 {
         push @results, info( NO_RESPONSE_SOA_QUERY => {} );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub syntax05
 
 sub syntax06 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my @nss;
     {
@@ -330,15 +452,38 @@ sub syntax06 {
     my %seen_rnames;
     for my $ns ( @nss ) {
 
+        if ( not Zonemaster::Engine::Profile->effective->get( q{net.ipv6} ) and $ns->address->version == $IP_VERSION_6 )
+        {
+            push @results,
+              info(
+                IPV6_DISABLED => {
+                    ns     => $ns->string,
+                    rrtype => q{SOA},
+                }
+              );
+            next;
+        }
+
+        if ( not Zonemaster::Engine::Profile->effective->get( q{net.ipv4} ) and $ns->address->version == $IP_VERSION_4 )
+        {
+            push @results,
+              info(
+                IPV4_DISABLED => {
+                    ns     => $ns->string,
+                    rrtype => q{SOA},
+                }
+              );
+            next;
+        }
+
         my $p = $ns->query( $zone->name, q{SOA} );
 
         if ( not $p ) {
             push @results,
               info(
                 NO_RESPONSE => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    dname   => $zone->name,
+                    ns    => $ns->string,
+                    dname => $zone->name,
                 }
               );
             next;
@@ -355,7 +500,7 @@ sub syntax06 {
         $rname =~ s/([^\\])[.]/$1@/smx;    # Replace first non-escaped dot with an at-sign
         $rname =~ s/[\\][.]/./smgx;        # Un-escape dots
         $rname =~ s/[.]\z//smgx;           # Validator does not like final dots
-        if ( not valid( $rname ) ) {
+        if ( not Email::Valid->address( $rname ) ) {
             push @results,
               info(
                 RNAME_RFC822_INVALID => {
@@ -440,12 +585,12 @@ sub syntax06 {
 
     } ## end for my $ns ( @nss )
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub syntax06
 
 sub syntax07 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my $p = $zone->query_one( $zone->name, q{SOA} );
 
@@ -458,12 +603,12 @@ sub syntax07 {
         push @results, info( NO_RESPONSE_SOA_QUERY => {} );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 }
 
 sub syntax08 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
 
     my $p = $zone->query_one( $zone->name, q{MX} );
 
@@ -598,9 +743,9 @@ sub check_name_syntax {
               );
         }
 
-    }
+    } ## end if ( $name ne q{.} )
 
-    if ( not scalar @results ) {
+    if ( not grep { $_->tag ne q{TEST_CASE_START} } @results ) {
         push @results,
           info(
             $info_label_prefix
@@ -631,9 +776,9 @@ Zonemaster::Engine::Test::Syntax - test validating the syntax of host names and
 
 Runs the default set of tests and returns a list of log entries made by the tests.
 
-=item translation()
+=item tag_descriptions()
 
-Returns a refernce to a hash with translation data. Used by the builtin translation system.
+Returns a refernce to a hash with translation functions. Used by the builtin translation system.
 
 =item metadata()
 
